@@ -131,6 +131,18 @@ from zim.signals import SignalEmitter, SIGNAL_AFTER
 
 logger = logging.getLogger('zim.fs')
 
+
+def adapt_from_newfs(file):
+	from zim.newfs import LocalFile, LocalFolder
+
+	if isinstance(file, LocalFile):
+		return File(file.path)
+	elif isinstance(file, LocalFolder):
+		return Dir(file.path)
+	else:
+		return file
+
+
 #: gobject and gio libraries are imported for optional features, like trash
 gobject = None
 gio = None
@@ -828,6 +840,7 @@ class UnixPath(object):
 		# Using shutil.move instead of os.rename because move can cross
 		# file system boundaries, while rename can not
 		logger.info('Rename %s to %s', self, newpath)
+		newpath = adapt_from_newfs(newpath)
 		if self.path == newpath.path:
 			raise AssertionError, 'Renaming %s to itself !?' % self.path
 
@@ -1661,6 +1674,7 @@ class UnixFile(FilePath):
 		destination is a folder, we will copy to a file below that
 		folder of the same name
 		'''
+		dest = adapt_from_newfs(dest)
 		assert isinstance(dest, (File, Dir))
 		if isinstance(dest, Dir):
 			assert not dest == self.dir, 'BUG: trying to copy a file to itself'
@@ -1878,19 +1892,21 @@ class FileHandle(file):
 if hasattr(os, 'replace'):
 	_replace_file = os.replace
 elif sys.platform == 'win32':
-	import win32api, win32con
+	# The win32api.MoveFileEx method somehow does not like our unicode,
+	# the ctypes version does ??!
+	import ctypes
+	_MoveFileEx = ctypes.windll.kernel32.MoveFileExW
+	_MoveFileEx.argtypes = [ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_uint32]
+	_MoveFileEx.restype = ctypes.c_bool
 	def _replace_file(src, dst):
-		# True atomic replace, if supported
 		try:
-			win32api.MoveFileEx(src, dst, win32con.MOVEFILE_REPLACE_EXISTING)
+			if not _MoveFileEx(src, dst, 1): # MOVEFILE_REPLACE_EXISTING
+				raise OSError, 'Could not replace "%s" -> "%s"' % (src, dst)
 		except:
 			# Sometimes it fails - we play stupid and try again...
 			time.sleep(0.5)
-			win32api.MoveFileEx(src, dst, win32con.MOVEFILE_REPLACE_EXISTING)
-
-		# Fall back using 2-file shuffle approach
-		#~ bak = dst + '.zim-bak~'
-		#~ win32api.ReplaceFile(src, dst, bak, 0)
+			if not _MoveFileEx(src, dst, 1): # MOVEFILE_REPLACE_EXISTING
+				raise OSError, 'Could not replace "%s" -> "%s"' % (src, dst)
 else:
 	_replace_file = os.rename
 
