@@ -27,6 +27,8 @@ from zim.gui.clipboard import \
 	INTERNAL_PAGELIST_TARGET_NAME, INTERNAL_PAGELIST_TARGET, \
 	pack_urilist, unpack_urilist
 
+from zim.actions import PRIMARY_MODIFIER_MASK
+
 
 logger = logging.getLogger('zim.gui.pageindex')
 
@@ -413,7 +415,7 @@ class PageTreeView(BrowserTreeView):
 		handled = False
 		#~ print 'KEY %s (%i)' % (gtk.gdk.keyval_name(event.keyval), event.keyval)
 
-		if event.state & gtk.gdk.META_MASK:
+		if event.state & PRIMARY_MODIFIER_MASK:
 			if event.keyval == KEYVAL_C:
 				self.emit('copy')
 				handled = True
@@ -490,10 +492,14 @@ class PageTreeView(BrowserTreeView):
 			dragcontext.finish(False, False, time) # NOK
 			return
 
-		if self.ui.do_move_page(source, dest, update_links=True):
-			dragcontext.finish(True, False, time) # OK
-		else:
+		try:
+			notebook = self.ui.notebook # XXX
+			notebook.move_page(source, dest, update_links=True)
+		except:
+			logger.exception('Failed to move page %s -> %s', source, dest)
 			dragcontext.finish(False, False, time) # NOK
+		else:
+			dragcontext.finish(True, False, time) # OK
 
 	def set_current_page(self, path, vivificate=False):
 		'''Select a page in the treeview
