@@ -7,11 +7,17 @@ from __future__ import with_statement
 
 from datetime import datetime
 
+import logging
+
+logger = logging.getLogger('zim.notebook.index')
+
 from zim.utils import natural_sort_key
 from zim.notebook.page import Path, HRef, \
 	HREF_REL_ABSOLUTE, HREF_REL_FLOATING, HREF_REL_RELATIVE
 from zim.tokenparser import TokenBuilder
 
+from zim.formats import ParseTreeBuilder
+
 from .base import *
 
 
@@ -28,11 +34,18 @@ PAGE_EXISTS_AS_LINK = 1 # placeholder for link target
 PAGE_EXISTS_HAS_CONTENT = 2 # either has content or children have content
 
 
+def emptyParseTree():
+	b = ParseTreeBuilder()
+	b.start('zim-tree')
+	b.end('zim-tree')
+	return b.get_parsetree()
+
+
 class PagesIndexer(IndexerBase):
 	'''Indexer for the "pages" table.
 
 	@signal: C{page-row-inserted (row)}: new row inserted
-	@signal: C{page-row-changed (row)}: row changed
+	@signal: C{page-row-changed (row, oldrow)}: row changed
 	@signal: C{page-row-deleted (row)}: row to be deleted
 
 	@signal: C{page-changed (row, content)}: page contents changed
@@ -40,7 +53,7 @@ class PagesIndexer(IndexerBase):
 
 	__signals__ = {
 		'page-row-inserted': (None, None, (object,)),
-		'page-row-changed':  (None, None, (object,)),
+		'page-row-changed':  (None, None, (object, object)),
 		'page-row-deleted':  (None, None, (object,)),
 		'page-changed':      (None, None, (object, object))
 	}
@@ -96,7 +109,13 @@ class PagesIndexer(IndexerBase):
 		if row is None:
 			self.insert_page(pagename, filerow['id'])
 		elif row['source_file'] is None:
-			self._set_source_file(pagename, filerow['id'])
+			self.db.execute(
+				'UPDATE pages SET source_file=?, mtime=?, is_link_placeholder=? WHERE name=?',
+				(filerow['id'], None, False, pagename.name)
+			)
+			self.update_parent(pagename.parent)
+			newrow = self._select(pagename)
+			self.emit('page-row-changed', newrow, row)
 		else:
 			# TODO: Flag conflict
 			raise NotImplementedError
@@ -128,7 +147,17 @@ class PagesIndexer(IndexerBase):
 
 		if row['source_file'] == filerow['id']:
 			if row['n_children'] > 0:
-				self._set_source_file(pagename, None)
+				self.db.execute(
+					'UPDATE pages SET source_file=?, mtime=? WHERE name=?',
+					(None, None, pagename.name)
+				)
+				self.update_parent(pagename, oldrow=row)
+					# checks if any children have sources - else will be removed
+				try:
+					row = self._select(pagename)
+					self.emit('page-changed', row, emptyParseTree())
+				except IndexNotFoundError:
+					pass
 			else:
 				self.remove_page(pagename)
 		else:
@@ -174,7 +203,7 @@ class PagesIndexer(IndexerBase):
 
 		return row['id']
 
-	def update_parent(self, parentname, allow_cleanup=lambda r: True):
+	def update_parent(self, parentname, allow_cleanup=lambda r: True, oldrow=None):
 		row = self._select(parentname)
 		assert row is not None
 
@@ -205,15 +234,10 @@ class PagesIndexer(IndexerBase):
 			if bool(row['is_link_placeholder']) is not is_placeholder:
 				self.update_parent(parentname.parent) # recurs
 
-			parentname = PageIndexRecord(self._select(parentname))
-
 			# notify others
 			if not parentname.isroot:
-				#if (row['n_children'] != n_children) \
-				#and (row['n_children'] == 0 or n_children == 0):
-				#	self.emit('page-row-has-child-changed', row)
-
-				self.emit('page-row-changed', row)
+				newrow = self._select(parentname)
+				self.emit('page-row-changed', newrow, oldrow or row)
 
 	def update_page(self, pagename, mtime, content):
 		self.db.execute(
@@ -223,21 +247,7 @@ class PagesIndexer(IndexerBase):
 
 		row = self._select(pagename)
 		self.emit('page-changed', row, content)
-		self.emit('page-row-changed', row)
-
-	def _set_source_file(self, pagename, file_id):
-		self.db.execute(
-			'UPDATE pages SET source_file=?, mtime=?, is_link_placeholder=? WHERE name=?',
-			(file_id, None, False, pagename.name)
-		)
-
-		if file_id is None:
-			# check any children have sources - else will be removed
-			self.update_parent(pagename)
-		else:
-			self.update_parent(pagename.parent)
-			row = self._select(pagename)
-			self.emit('page-row-changed', row)
+		self.emit('page-row-changed', row, row)
 
 	def remove_page(self, pagename, allow_cleanup=lambda r: True):
 		# allow_cleanup is used by LinksIndexer when cleaning up placeholders
@@ -391,8 +401,19 @@ class PagesViewInternal(object):
 				else:
 					return self.resolve_pagename(start.parent, href.parts())
 
-
 	def resolve_pagename(self, parent, names):
+		page_id, pagename, branch = self._resolve_pagename(parent, names)
+		if page_id is None:
+			try:
+				page_id = self.get_page_id(pagename)
+			except IndexNotFoundError:
+				pass
+			else:
+				logger.error('BUG: issue #19 seen: (%r, %r) -> (None, %r) - branch %i !', parent, names, pagename, branch)
+
+		return page_id, pagename
+
+	def _resolve_pagename(self, parent, names):
 		'''Resolve a pagename in the right case'''
 		# We do not ignore placeholders here. This can lead to a dependencies
 		# in how links are resolved based on order of indexing. However, this
@@ -428,9 +449,9 @@ class PagesViewInternal(object):
 					pagename = Path(row['name'])
 					page_id = row['id']
 				else: # no match
-					return None, pagename.child(':'.join(names[i:]))
+					return None, pagename.child(':'.join(names[i:])), 1
 		else:
-			return page_id, pagename
+			return page_id, pagename, 2
 
 	def walk(self, parent_id):
 		# Need to do this recursive to preserve sorting
@@ -538,7 +559,7 @@ class PagesView(IndexView):
 			'SELECT parent FROM pages WHERE name=?', (path.name,)
 		).fetchone()
 		if r is None:
-			raise IndexNotFoundError, 'No such page: %s', path
+			raise IndexNotFoundError, 'No such page: %s' % path
 		else:
 			parent_id = r[0]
 
@@ -578,7 +599,7 @@ class PagesView(IndexView):
 			'SELECT * FROM pages WHERE name=?', (path.name,)
 		).fetchone()
 		if r is None:
-			raise IndexNotFoundError, 'No such page: %s', path
+			raise IndexNotFoundError, 'No such page: %s' % path
 
 		if r['n_children'] > 0:
 			r = self.db.execute(
@@ -738,7 +759,7 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 			treeiter = self.get_iter(parent.treepath) # not mytreeiter !
 			self.emit('row-has-child-toggled', parent.treepath, treeiter)
 
-	def on_page_row_changed(self, o, row):
+	def on_page_row_changed(self, o, row, oldrow):
 		# no clear cache here - just update row
 		for treepath in self._find_all_pages(row['name']):
 			treeiter = self.get_iter(treepath) # not mytreeiter !
