@@ -184,7 +184,7 @@ class PluginManager(ConnectorMixin, collections.Mapping):
 		# List "zim.plugins" sub modules based on __path__ because this
 		# parameter determines what folders will considered when importing
 		# sub-modules of the this package once this module is loaded.
-		plugins = set(['arithmetic', 'attachmentbrowser', 'backlinkpane', 'bookmarksbar', 'calendar', 'diagrameditor', 'distractionfree', 'ditaaeditor', 'equationeditor', 'gnu_r_ploteditor', 'gnuplot_ploteditor', 'inlinecalculator', 'insertsymbol', 'linesorter', 'linkmap', 'printtobrowser', 'quicknote', 'scoreeditor', 'screenshot', 'sequencediagrameditor', 'sourceview', 'spell', 'tableeditor', 'tableofcontents', 'tags', 'tasklist', 'trayicon', 'versioncontrol', 'zeitgeist-logger']) # DEFAULT PLUGINS COMPILED IN BY SETUP.PY
+		plugins = set(['arithmetic', 'attachmentbrowser', 'backlinkpane', 'bookmarksbar', 'calendar', 'diagrameditor', 'distractionfree', 'ditaaeditor', 'equationeditor', 'gnu_r_ploteditor', 'gnuplot_ploteditor', 'inlinecalculator', 'insertsymbol', 'linesorter', 'linkmap', 'osx_menubar', 'printtobrowser', 'quicknote', 'scoreeditor', 'screenshot', 'sequencediagrameditor', 'sourceview', 'spell', 'tableeditor', 'tableofcontents', 'tags', 'tasklist', 'trayicon', 'versioncontrol', 'zeitgeist-logger']) # DEFAULT PLUGINS COMPILED IN BY SETUP.PY
 		for dir in __path__:
 			dir = Dir(dir)
 			for candidate in dir.list(): # returns [] if dir does not exist
@@ -759,6 +759,10 @@ class WindowExtension(ObjectExtension):
 
 		if hasattr(self, 'uimanager_xml'):
 			actiongroup = get_gtk_actiongroup(self)
+			if hasattr(self, 'uimanager_menu_labels'):
+				actiongroup.add_actions(
+					sorted((k, None, v) for k, v in self.uimanager_menu_labels.items())
+				)
 			self.window.uimanager.insert_action_group(actiongroup, 0)
 			self._uimanager_id = self.window.uimanager.add_ui_from_string(self.uimanager_xml)
 
