@@ -142,10 +142,17 @@ class TaskListDialog(TaskListWidgetMixin, Dialog):
 
 		# TODO: use menu button here and add same options as in context menu
 		#       for filtering the list
+		def on_show_active_toggle(o):
+			active = self.act_toggle.get_active()
+			if self.uistate['only_show_act'] != active:
+				self.uistate['only_show_act'] = active
+				self.task_list.set_filter_actionable(active)
+
 		self.act_toggle = gtk.CheckButton(_('Only Show Active Tasks'))
 			# T: Checkbox in task list - this options hides tasks that are not yet started
 		self.act_toggle.set_active(self.uistate['only_show_act'])
-		self.act_toggle.connect('toggled', lambda o: self.task_list.set_filter_actionable(o.get_active()))
+		self.act_toggle.connect('toggled', on_show_active_toggle)
+		self.uistate.connect('changed', lambda o: self.act_toggle.set_active(self.uistate['only_show_act']))
 		hbox.pack_start(self.act_toggle, False)
 
 		# Statistics label
@@ -171,12 +178,14 @@ class TaskListDialog(TaskListWidgetMixin, Dialog):
 			# now it is at least on idle
 
 		### XXX HACK to get dependency to connect to
+		###   -- no access to plugin, so can;t use get_extension()
+		##    -- duplicat of this snippet in MainWindowExtension
 		for e in window.ui.notebook.__zim_extension_objects__:
 			if hasattr(e, 'indexer') and e.indexer.__class__.__name__ == 'TasksIndexer':
-				self.connectto(e.indexer, 'tasklist-changed', callback)
+				self.connectto(e, 'tasklist-changed', callback)
 				break
 		else:
-			raise AssertionError, 'Could not find indexer'
+			raise AssertionError, 'Could not find tasklist notebook extension'
 
 	def do_response(self, response):
 		self.uistate['hpane_pos'] = self.hpane.get_position()
@@ -675,7 +684,10 @@ class TaskListTreeView(BrowserTreeView):
 			# And finally the filter string should match
 			# FIXME: we are matching against markup text here - may fail for some cases
 			inverse, string = self.filter
-			match = string in description or string in pagename
+			if string.startswith('@'):
+				match = string[1:].lower() in [t.lower() for t in tags]
+			else:
+				match = string in description or string in pagename
 			if (not inverse and not match) or (inverse and match):
 				visible = False
 
@@ -725,7 +737,7 @@ class TaskListTreeView(BrowserTreeView):
 		if due != _MAX_DUE_DATE:
 			text += ['<b>', _('Due'), ':</b> ', due, '\n'] # T: due date for task
 
-		text += ['<b>', _('Page'), ':</b> ', page] # T: page label
+		text += ['<b>', _('Page'), ':</b> ', encode_markup_text(page)] # T: page label
 
 		tooltip.set_markup(''.join(text))
 		return True
